/**************************************************************************************
Author: 		Dave Szybala
Date: 			11/19/2009
Program: 		Material_Library.ms
Description: 	This script will provide special options for the materail library.
***************************************************************************************/

try (destroyDialog matLibraryRollout) catch()
	
(--put script into local scope
	global matLibraryRollout
	local matLibraryIniFile = getDir #plugcfg + "\\MaterialLibrary.ini"
	local matLibraryRolloutPos
	
	/**************************************************************************************/
	rollout matLibraryRollout "Material Library"
	(
		fn Pick_Cam_Filter currentObj = isKindOf currentObj camera
		fn Pick_Obj_Filter currentObj = superClassOf currentObj == GeometryClass
		
		button loadMatsBtn "Load Material Library" pos:[5,5] width:170 height:20
		button saveMatsBtn "Save Materials to File" pos:[5,30] width:170 height:20
		button appendMatsBtn "Append Materials to Library" pos:[5,55] width:170 height:20
		button clearMatsBtn "Clear Library Materials" pos:[5,80] width:170 height:20
		button applyMatsBtn "All Library Materials to Scene" pos:[5,105] width:170 height:20
		
		groupbox thumbGroup "Render" pos:[5,135] width:170 height:115
		pickbutton camPickBtn "Pick Camera" pos:[15,155] width:90 height:20 filter:Pick_Cam_Filter
		pickbutton objPickBtn "Pick Object" pos:[15,180]width:90 height:20 filter:Pick_Obj_Filter
		spinner widthSpn "W" pos:[115,157] range:[10,2048,256] type:#integer scale:1 width:50 height:15 
		spinner heightSpn "H" pos:[115,182] range:[10,2048,256] type:#integer scale:1 width:50 height:15
		button thumbBtn "Render Now" pos:[15,205]width:150 height:20 enabled:false
		progressbar progBar pos:[15,233] width:150 height:10
		
		on matLibraryRollout moved pos do
			setIniSetting matLibraryIniFile "Dialog" "Position" (pos as string)
			
		-------------------------------------------------------------------
		on loadMatsBtn pressed do
		(
			local libraryPath = getOpenFileName caption:"Material Library File" types:("Material Library (*.mat)|*.mat")
			
			if libraryPath != undefined do
			(
				if (loadMaterialLibrary libraryPath) then
					print "Material library has been loaded."
				else
					print ("ERROR: Could not load " + libraryPath + ".")
			)--end if
		)--end on
		
		-------------------------------------------------------------------
		on saveMatsBtn pressed do
		(
			local savePath = getSaveFileName caption:"Save File Name" types:("Material Library (*.mat)|*.mat")
			
			if savePath != undefined do
			(
				if currentMaterialLibrary.count == 0 do
				(
					for mat in sceneMaterials do
					(
						if superClassOf mat == material do append currentMaterialLibrary mat
					)--end for
				)--end if
				
				saveMaterialLibrary savePath
				
				print "Materials have been saved."
			)--end if
		)--end on
	
		-------------------------------------------------------------------
		on appendMatsBtn pressed do
		(
			local testMat = 0
			local testName = 0
			
			for mat in sceneMaterials do
			(
				if superClassOf mat == material do
				(
					testMat = findItem currentMaterialLibrary mat
				
					for i = 1 to currentMaterialLibrary.count do
					(
						if mat.name == currentMaterialLibrary[i].name do testName = 1
					)--end for
				
					if testMat == 0 and testName == 0 do append currentMaterialLibrary mat
					testName = 0
				)--end if
			)--end for
			
			print "Materials have been appended."
		)--end on
	
		-------------------------------------------------------------------
		on clearMatsBtn pressed do
		(
			while currentMaterialLibrary.count > 0 do
			(
				for mat in currentMaterialLibrary do deleteItem currentMaterialLibrary mat.name
			)--end while
			
			print "Library has been cleared."
		)--end on
		
		-------------------------------------------------------------------
		on applyMatsBtn pressed do
		(
			local boxXOffset = 0
			local boxYOffset = 0
			local counter = 1
			local currentLayer
			local currentLayerName = "Material_Library"
			
			currentLayer = LayerManager.getLayerFromName currentLayerName
			while currentLayer != undefined do
			(
				currentLayerName = currentLayerName + (counter as string)
				counter += 1
				currentLayer = LayerManager.getLayerFromName currentLayerName
			)--end while
			
			currentLayer = LayerManager.newLayerFromName currentLayerName
			currentLayer.current = true
			
			for current in currentMaterialLibrary do
			(
				b = box()
				b.renderable = false
				b.length = b.height = b.width = 2
				b.material = current
				b.name = b.material.name + "_Box"
				move b [boxXOffset,boxYOffset,0]
				
				if boxXOffset == 40 then
				(
					boxYOffset += 4
					boxXOffset = 0
				)--end if
				else
					boxXOffset += 4
			)--end for
			
			print "All library materials added to scene."
		)--end on
		
		-------------------------------------------------------------------
		on camPickBtn picked currentObj do
		(
			if currentObj != undefined do
			(
				camPickBtn.text = currentObj.name
				
				if objPickBtn.object != undefined do
					thumbBtn.enabled = true
			)--end if
		)--end on
		
		-------------------------------------------------------------------
		on objPickBtn picked currentObj do
		(
			if currentObj != undefined do
			(
				objPickBtn.text = currentObj.name
				
				if camPickBtn.object != undefined then
					thumbBtn.enabled = true
			)--end if
		)--end on
		
		-------------------------------------------------------------------
		on thumbBtn pressed do
		(
			local percent = 0
			local currentCam = camPickBtn.object
			local currentObject = objPickBtn.object
			local currentBitmap = undefined
			local saveLocation = getSavePath caption:"Thumbnail Save Location"
			local vrayFBState = false
			
			if currentCam != undefined and currentObject != undefined and saveLocation != undefined do
			(
				progBar.color = [200,0,0]
				progBar.value = 0
				
				if (matchPattern ((classOf renderers.current) as string) pattern:"V_Ray_Adv*" ignorecase:true) do
				(
					vrayFBState = renderers.current.output_on
					renderers.current.output_on = false
				)--end if
				
				for index = 1 to sceneMaterials.count do
				(
					if superClassOf sceneMaterials[index] == material do
					(
						currentObject.material = sceneMaterials[index]
						
						currentBitmap = bitmap widthSpn.value heightSpn.value gamma:(0.454545)
						render camera:currentCam to:currentBitmap
						currentBitmap.filename = (saveLocation + "\\" + sceneMaterials[index].name + ".jpg")
						save currentBitmap
						close currentBitmap
						gc light:true
					)--end if
					
					percent = 100.0 * index / sceneMaterials.count
					progBar.value = percent
					if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
				)--end for
				
				progBar.color = [0,200,0]
				progBar.value = 100
				
				if (matchPattern ((classOf renderers.current) as string) pattern:"V_Ray_Adv*" ignorecase:true) do
					renderers.current.output_on = vrayFBState
			)--end if
		)--end on
	)--end rollout
	matLibraryRolloutPos = execute (getIniSetting matLibraryIniFile "Dialog" "Position")
	if matLibraryRolloutPos == OK do matLibraryRolloutPos = [100,100]
	if matLibraryRolloutPos.x > sysinfo.desktopsize.x - 180 do matLibraryRolloutPos.x = sysinfo.desktopsize.x - 185
	if matLibraryRolloutPos.y > sysinfo.desktopsize.y - 315 do matLibraryRolloutPos.y = sysinfo.desktopsize.y - 315
	if matLibraryRolloutPos.x < 0 do matLibraryRolloutPos.x = 0 
	if matLibraryRolloutPos.y < 0 do matLibraryRolloutPos.y = 0
	createDialog matLibraryRollout 180 255 matLibraryRolloutPos.x matLibraryRolloutPos.y
)--end local scope